%
% script para un codigo de diferencias finitas en coordenadas cartesianas en 2D 
%

global data

% la variable initia dice si el calculo por iniciar arranca de cero o es
% continuacion de otro preexistente en memoria.
% initia = 1 : arranca de cero , no hay registro anterior en memoria
% initia = 0 : continuacion de una corrida anterior

if(exist('initia')==0),
    disp('Enter variable INITIA ')
    disp(' initia == 0 ---> if is a continuation of an older run  ')
    disp(' initia ~= 0 ---> if started from the beginning  ')
    initia = input(' Initia = ')
end

% Generacion de una grilla de puntos en dominio rectangular
Nx = 40; Ny = 40;
x1 = (0:Nx-1)'/(Nx-1);
x2 = (0:Ny-1)'/(Ny-1);
[xnod,icone] = qq3d([x1,0*x1],[0*x2,x2]);
[numnp,ndm]=size(xnod);

% parametros, ver la version 1D para mas detalles
dt = 1e15;
ntime = 2;
conductivity = 1;
rho = 1;
Cp = 1;
ttime = 0;
ndf = 1;
maxit = 5;
tol_newton = 1e-10;
scaling = 1;
nfreqplot = 1;

numnp = size(xnod,1);  % numeros de nodos en la malla 2D
% nodal source term
source = -2*ones(numnp,1);
%source = 0*xnod.^3;

% aqui se calcula los nodos y segmentos del contorno
% nodes_bou es una arreglo que tiene una matriz de 2 columnas
% donde cada fila de esa matriz representa un segmento del contorno
tol  = 1e-6;
[ifaceb,nodes_bou] = pfbou2d(xnod,icone,1);

% establecemos las fijaciones (ver el caso 1D)
fixa = zeros(numnp,2);
xmin = min(xnod); xmax = max(xnod);
% diferenciamos los nodos que estan a derecha, izquierda, arriba y abajo
nodes_lef = find(abs(xnod(:,1)-xmin(1))<tol);
nodes_rig = find(abs(xnod(:,1)-xmax(1))<tol);
nodes_bot = find(abs(xnod(:,2)-xmin(2))<tol);
nodes_top = find(abs(xnod(:,2)-xmax(2))<tol);

% calculamos la normal a cada nodo del contorno
[slip_data] = get_normal(xnod,icone,nodes_bou,1);
normal_node(slip_data(:,1),1:ndm) = slip_data(:,2:ndm+1);

% de acuerdo al problema en cuestion activamos una porcion de este gran IF
% es decir, aqui hay varios ejemplos en una misma rutina. Si uno quisiera
% podria dividir los ejemplos para que queden mas compacto y mas legible.
%

if 1
    % todo el contorno fijo
    nodes_fixed = rmdblist(m2v(nodes_bou));
    fixa(nodes_fixed,1)=1;
    if 1 % activar la fuente para que phi=x.^2 sea solucion
        fixa(nodes_fixed,2)=xnod(nodes_fixed,1).^2;
    else
        fixa(nodes_fixed,2)=1;
    end
elseif 0
    %    nodes_fixed = rmdblist([nodes_lef;nodes_rig;nodes_bot]);
    nodes_fixed = rmdblist([nodes_lef;nodes_top;nodes_bot]);
    fixa([nodes_lef],1)=1;
    fixa([nodes_lef],2)=0;
    fixa([nodes_rig],1)=0;
    fixa([nodes_rig],2)=2*xnod(nodes_rig-1,1);    
    fixa([nodes_top],1)=1;
    fixa([nodes_top],2)=xnod(nodes_top,1).^2;    
    fixa([nodes_bot],1)=1;
    fixa([nodes_bot],2)=xnod(nodes_bot,1).^2;        
elseif 1 % 2 extremos fijos y los otros 2 libres
    if 0
        nodes_fixed = rmdblist([nodes_lef;nodes_rig]);
        fixa(nodes_lef,1)=1;
        fixa(nodes_lef,2)=0;
        fixa(nodes_rig,1)=1;
        fixa(nodes_rig,2)=1;
    else
        nodes_fixed = rmdblist([nodes_top;nodes_bot]);
        fixa(nodes_top,1)=1;
        fixa(nodes_top,2)=0;
        fixa(nodes_bot,1)=1;
        fixa(nodes_bot,2)=1;
    end
elseif 1
    nodes_fixed = rmdblist([nodes_lef]);
    fixa(nodes_bot,2)=-1;
    fixa(nodes_top,2)=1;    
    fixa(nodes_rig,2)=0;
    fixa(nodes_fixed,1)=1;
    fixa(nodes_fixed,2)=0;
end

% initial state
if initia
    state = 0*xnod(:,1);
    state(nodes_fixed,1) = fixa(nodes_fixed,2);
    R = zeros(numnp,1);
end

% handle data (Empaquetamos los datos)
data.param.dt = dt;
data.param.rho = rho;
data.param.Cp = Cp;
data.param.conductivity = conductivity;
data.param.ntime = ntime;
data.param.type_solver = 'implicit';
data.param.relax = 1;
data.param.ndf = ndf;
data.param.scaling = scaling;
data.param.nfreqplot = nfreqplot;

data.xnod = xnod;
data.fixa = fixa;
data.icone = icone;
data.Nx = Nx;
data.Ny = Ny;
data.nodes_bou = nodes_bou;
data.normal_node = normal_node;

data.source = source;
data.state = state;

data.pre_step = 1;

% Calculo propiamente dicho, lazo temporal
for itime=1:ntime
    check_conv = 1;
    iter = 1;
    data.state_old = data.state;
    % iteraciones del Newton en caso que el problema sea no lineal
    while check_conv
        ttime = ttime+dt;
        % ensamblaje de la matriz y del residuo
        [K,R] = fdm_assemble();
        % resolucion del sistema, basado en incrementos
        [phi_new,R] = fdm_solver(data,K,R);
        % update solution
        data.state = phi_new;
        % convergence check
        disp([' time step = ' num2str(itime) ' t= ' num2str(ttime) ' | R | = ' num2str(norm(R))])
        check_conv = (norm(R)>tol_newton)&(iter<=maxit);
        iter = iter+1;
    end
    % plot (graficacion)
    fdm_plot(data,itime)
end

